// Copyright 2012-2015, Jean Rene Dawin
//
// This Library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 2.1 of the License, or
// (at your option) any later version.
//
// This Library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this Library.  If not, see <http://www.gnu.org/licenses/>.

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "escwyp_chain.h"


void chainConstructor(){


}

void chain_default_params(struct chain_params *the_params){

    the_params->last_chain_value = 0.0;
    the_params->sined_chain_pos = 0.0;
    the_params->chain_delta = 0.2;
    the_params->sined_chain_freq = 500.0;

}

void print_params_chain(struct chain_params *the_params){

   if(the_params != NULL){

      fprintf(stdout, "-----------------\n");
      fprintf(stdout, "last_chain_value %f\n", the_params->last_chain_value );
      fprintf(stdout, "sined_chain_pos %f\n", the_params->sined_chain_pos);
      fprintf(stdout, "-----------------\n");
   }else{
      fprintf(stdout, "Pardon, no params set.\n");
   }
}


void chain_refresh_parameters(){


}

void chain_set_params(struct chain_params *params){

   the_params = params;

}

void chain_set_freq(float freq){

    fprintf(stdout, "AHJA (freq=%f))\n", freq);
    if(chain_mode == 0){
        // TODO: check for division through zero!
        the_params->chain_delta = freq;
    }else{
        // TODO: check for division through zero!
        the_params->chain_delta = freq;
        the_params->sined_chain_freq = freq;
    }
}

void chain_set_freq_from_klavier(const float freq, const float low_freq, const float high_freq){

    fprintf(stdout, "AHJA %f (freq=%f)(low=%f, high=%f)\n", (freq-low_freq)/(high_freq-low_freq), freq, low_freq, high_freq);
    if(chain_mode == 0){
        // TODO: check for division through zero!
        if(!self_control){
           the_params->chain_delta = 0.001 + (freq-low_freq)/(high_freq-low_freq)*(1.0 - 0.001);
        }else{
           the_params->chain_delta = chain_delta;
        }
    }else{
        // TODO: check for division through zero!
        if(!self_control){
           the_params->chain_delta = 0.001 + (freq-low_freq)/(high_freq-low_freq)*(1.0 - 0.001);
           the_params->sined_chain_freq = freq;
        }else{
           the_params->chain_delta = chain_delta;
           the_params->sined_chain_freq = sined_chain_freq;
	}
    }
    fprintf(stdout, "CHAINDELTA = %f\n", the_params->chain_delta);
}


void chain_init_values(){

    the_params->last_chain_value = 0.0;
    the_params->sined_chain_pos = 0.0;
    the_params->chain_delta = 0.2;
    the_params->sined_chain_freq = 500.0;
}

void makeChains(const int chainLength, float *buff_L, const unsigned int srate)
{

    if(chain_mode == 0){
        makeChain(chainLength, buff_L);
    }else{
        makeChainSined(chainLength, buff_L, srate);
    }
}


// This function is called when button "Key" is pressed. ----------------
void makeChain(const int chainLength, float *buff_L)
{

   //srand(time(0));

   int kett_high = 1;
   int kett_low = -1;
   buff_L[0] = (float) the_params->last_chain_value;
   //fprintf(stdout, "0 %f\n", buff_L[0]);
   int currentLength = 1;

   while( currentLength < chainLength ){

      float rndaNex = (float) the_params->chain_delta*random()/RAND_MAX;
      float rnd_help = (float) random()/RAND_MAX;
      int sign = 0;
      if(rnd_help < 0.5){
         sign = -1;
      }else{
         sign = 1;
      }

      float possible_rndaNex = buff_L[currentLength-1] + sign*rndaNex;

      if( possible_rndaNex > kett_high || possible_rndaNex < kett_low ){
         rndaNex = buff_L[currentLength-1] - sign*rndaNex;
      }else{
         rndaNex = possible_rndaNex;
      }

      buff_L[currentLength] = rndaNex;
      //fprintf(stdout, "%i %f\n", currentLength, buff_L[currentLength]);
      currentLength++;
   }
   the_params->last_chain_value = (float) buff_L[chainLength-1];

}
// -------------------------------------------------------

// This function is called when button "Key2" is pressed. ----------------
void makeChainSined(const int chainLength, float *buff_L, const unsigned int srate)
{

   //srand(time(0));
   unsigned int sample_rate = srate;
   if(sample_rate == 0){
      sample_rate = 48000;
   }

   buff_L[0] = (float) the_params->last_chain_value;
   //fprintf(stdout, "0 %f\n", buff_L[0]);
   int currentLength = 1;
   while( currentLength < chainLength ){


      float kett_high = (float) 0.15*sinf(2.0 * M_PI * the_params->sined_chain_pos) + 0.85;
      float kett_low = (float) 0.15*sinf(2.0 * M_PI * the_params->sined_chain_pos) - 0.85;
      the_params->sined_chain_pos += the_params->sined_chain_freq / sample_rate;

      if(the_params->sined_chain_pos == 1.0){
         the_params->sined_chain_pos = 0.0; // chek!
      }

      float rndaNex = (float) the_params->chain_delta*random()/RAND_MAX;
      float rnd_help = (float) random()/RAND_MAX;
      int sign = 0;
      if(rnd_help < 0.5){
         sign = -1;
      }else{
         sign = 1;
      }

      float possible_rndaNex = buff_L[currentLength-1] + sign*rndaNex;

      if( possible_rndaNex > kett_high || possible_rndaNex < kett_low ){
         rndaNex = buff_L[currentLength-1] - sign*rndaNex;
      }else{
         rndaNex = possible_rndaNex;
      }

      buff_L[currentLength] = rndaNex;

      //buff_L[currentLength] = kett_high;
      //fprintf(stdout, "%i %f\n", currentLength, buff_L[currentLength]);
      currentLength++;

   }
   the_params->last_chain_value = (float) buff_L[chainLength-1];

}
// -------------------------------------------------------
//-------------------------------------------------------------

// getters
float get_chain_delta(){

    return chain_delta;

}

float get_sined_chain_freq(){

    return sined_chain_freq;
}

int get_chain_mode(){

    return chain_mode;

}

// setters
void set_chain_delta(const float delta){

    chain_delta = delta;

}

void set_sined_chain_freq(const float freq){

    sined_chain_freq = freq;

}

void set_chain_mode(const int mode){

    chain_mode = mode;

}

void chain_set_self_control(const int value)
{

    self_control = value;

}
int chain_get_self_control()
{

    return self_control;

}

