// Copyright 2015, Jean Rene Dawin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version. See License-GPL-2.txt.

#include "escwyp_chain_gui_elm.h"
#include "escwyp_chain_extern.h"
#include "escwyp_globals_extern.h"
#include <Elementary.h>

void chainPackGui(Evas_Object *box){

    elm_box_pack_end(box, chain_mode_check);
    evas_object_show(chain_mode_check);

    elm_box_pack_end(box , chain_delta_slider);
    evas_object_show(chain_delta_slider);

    elm_box_pack_end(box , sined_chain_freq_slider);
    evas_object_show(sined_chain_freq_slider);

}

void chainUnpackGui(Evas_Object *box){

    elm_box_unpack(box, chain_mode_check);
    evas_object_hide(chain_mode_check);

    elm_box_unpack(box , chain_delta_slider);
    evas_object_hide(chain_delta_slider);

    elm_box_unpack(box , sined_chain_freq_slider);
    evas_object_hide(sined_chain_freq_slider);

}

void chainInitGuiElm(const int hidden){

   // the box containing all of the following
   chain_base_box = elm_box_add(win);
   //evas_object_size_hint_weight_set(box2, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   //evas_object_size_hint_align_set(box2, EVAS_HINT_FILL, EVAS_HINT_FILL);
   //elm_box_align_set(box2, 0.5, 0.0);
   //elm_box_horizontal_set(distrib_base_box, EINA_TRUE);
   if(hidden == 0){
       //elm_box_pack_end(change_box, chain_base_box);
       //evas_object_show(chain_base_box);
   }else{
       //evas_object_show(chain_base_box);
       //evas_object_hide(chain_base_box);
   }

   // the ckeck-box setting mode to chain or sined_chain 
   chain_mode_check =  elm_check_add(win);
   elm_check_state_set(chain_mode_check, EINA_FALSE);
   elm_object_part_text_set(chain_mode_check, NULL, "Sined!");
   evas_object_smart_callback_add(chain_mode_check, "changed", chain_mode_check_fun, NULL);    

   // the slider associated with the chain_delta parameter in function makeChain called by button "Key"
   chain_delta_slider = elm_slider_add(win);
   evas_object_size_hint_align_set(chain_delta_slider, EVAS_HINT_FILL, 0.5);
   evas_object_size_hint_weight_set(chain_delta_slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   elm_slider_unit_format_set(chain_delta_slider, "%1.3f");
   elm_slider_indicator_show_set(chain_delta_slider, EINA_FALSE);
   elm_slider_min_max_set(chain_delta_slider, 0.001, 1.0);
   elm_slider_value_set(chain_delta_slider, 0.2);
   elm_slider_span_size_set(chain_delta_slider, 200);
   evas_object_smart_callback_add(chain_delta_slider, "delay,changed", chain_delta_slider_fun, NULL);    
   elm_object_disabled_set(chain_delta_slider, EINA_TRUE);

   // the slider associated with the sined_chain_freq parameter in function makeChainSined called by button "Key2"
   sined_chain_freq_slider = elm_slider_add(win);
   evas_object_size_hint_align_set(sined_chain_freq_slider, EVAS_HINT_FILL, 0.5);
   evas_object_size_hint_weight_set(sined_chain_freq_slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   elm_slider_unit_format_set(sined_chain_freq_slider, "%1.2f");
   elm_slider_indicator_show_set(sined_chain_freq_slider, EINA_FALSE);
   elm_slider_min_max_set(sined_chain_freq_slider, 20.0, 10000.0);
   elm_slider_value_set(sined_chain_freq_slider, 20.0);
   elm_slider_span_size_set(sined_chain_freq_slider, 200);
   evas_object_smart_callback_add(sined_chain_freq_slider, "delay,changed", sined_chain_freq_slider_fun, NULL);    
   elm_object_disabled_set(sined_chain_freq_slider, EINA_TRUE);

}

static void chain_mode_check_fun(void *data, Evas_Object *obj, void *event_info){

   fprintf(stdout, "Yis chain_mode_check_fun\n");
   int mode = elm_check_state_get(obj);
   set_chain_mode(mode);
   fprintf(stdout, "CHAINMODE %i\n", mode);

}

static void chain_delta_slider_fun(void *data, Evas_Object *obj, void *event_info){

   fprintf(stdout, "Yis chain_delta_slider_fun\n");
   float delta = (float) elm_slider_value_get(obj);
   set_chain_delta(delta);
   fprintf(stdout, "CHAINDELTA %f\n", delta);

}

static void sined_chain_freq_slider_fun(void *data, Evas_Object *obj, void *event_info){

   fprintf(stdout, "Yis sined_chain_freq_slider_fun\n");
   float freq = (float) elm_slider_value_get(obj);
   set_sined_chain_freq(freq);
   fprintf(stdout, "SINEDCHAINFREQ %f\n", freq);

}

void chain_delta_slider_set(const float value){

	elm_slider_value_set(chain_delta_slider, (double) value);	

}

void chain_freq_slider_set(const float value){

	elm_slider_value_set(sined_chain_freq_slider, (double) value);	

}

void chain_mode_check_set(const int value){

	elm_check_state_set(chain_mode_check, value);	

}
void chain_gui_set_self_control(const int value)
{
    chain_set_self_control(value);
    if(chain_get_self_control()){
   	elm_object_disabled_set(chain_delta_slider, EINA_FALSE);
   	elm_object_disabled_set(sined_chain_freq_slider, EINA_FALSE);
    }else{
   	elm_object_disabled_set(chain_delta_slider, EINA_TRUE);
   	elm_object_disabled_set(sined_chain_freq_slider, EINA_TRUE);
    }
}

