// Copyright 2015, Jean Rene Dawin
//
// This Library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 2.1 of the License, or
// (at your option) any later version.
//
// This Library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this Library.  If not, see <http://www.gnu.org/licenses/>.

#include <stdio.h>
#include <math.h>
//#include "escwyp_globals_extern.h"
#include "escwyp_distribs.h"


void makeDistri(const int distrib_size, const int low, const int high, const float center_fact, const float height, float *distrib, float *dist_marks, float *distrib_minimum, float *distrib_maximum ){

   //int number = high -low;
   int number = high < low ? low-high : high-low;
   int distrib_center_scalar = (int) roundf(center_fact*number);
   float summ1 = 0;
   float summ2 = 0.0;
   float mult_center = 0.0;
   float mult_remains = 0.0;
   float values[number];

   float tmp_counter = 0.0;


   int i;
   for(i = 0; i < distrib_size; i++){
      distrib[i] = 0.0;
      if(i < high){
         dist_marks[i] = 0.0;
      }else{
         dist_marks[i]  = 1.0;
      }
   }

   for(i = 1; i < number+1; i++){

      //values[i-1] = (float) i*i*i*i*i;
      //summ1 += (float) i*i*i*i*i;

      //values[i-1] = (float) log(i);
      //summ1 += (float) log(i);
      values[i-1] = (float) i;
      summ1 += (float) i;
   }
   for(i = 0; i < number; i++){
      values[i] /= summ1;
   }
   mult_center = height/values[number-1];
   for(i = 0; i < number; i++){
      values[i] *= mult_center;
      if(i <= number-2){
         summ2 += values[i];
      }
   } 
   mult_remains = (1-height)/summ2;
   for(i = 0; i < number-1; i++){

      values[i] *= mult_remains;
   }

   distrib[distrib_center_scalar] = values[number-1];
   i  = number-2;
   int distancer = 1;
   int distancel = 1;
   int flagy = 0;

   while(i >= 0 && flagy < 2){
      //fprintf(stdout, "preA %i %f\n", i, values[i]);
      if(distrib_center_scalar + distancer < number){
         distrib[distrib_center_scalar+distancer] = values[i];
         distancer++;
         i--;
         //fprintf(stdout, "A %i %f %i\n", i, values[i], distancel+distancer);
      }
      //fprintf(stdout, "B %i %f\n", i, values[i]);
      if(distrib_center_scalar - distancel >= 0){
         distrib[distrib_center_scalar-distancel] = values[i];
         distancel++;
         i--;
         //fprintf(stdout, "C %i %f %i\n", i, values[i], distancel+distancer);
      }

      if( i <= 0){ flagy++;}

   }

   //int j = 0;
   *distrib_maximum = 0.0;
   *distrib_minimum = 1.0;
   //for(i = low; i < high; i++){
   for(i = 0; i < distrib_size; i++){
      if(distrib[i] > *distrib_maximum){
         *distrib_maximum = distrib[i];
      }
      if(distrib[i] < *distrib_minimum && distrib[i] > 0.0){
         *distrib_minimum = distrib[i];
      }
      tmp_counter += distrib[i];
      dist_marks[i] = tmp_counter;
      //fprintf(stdout, "%f\n", distrib[i]);
      //fprintf(stdout, "%f\n", tmp_counter);
      //fprintf(stdout, "2mark: %i, acuu: %f, adder: %f\n", dist_marks[j], tmp_counter, distrib[i]);
      //fprintf(stdout, "%i\n", dist_marks[j]);
   }
   //fprintf(stdout, "%i\n", dist_marks[20]);


}

void makeEqualDistribution(const int distrib_size, const int low, const int high, float *distrib, float *dist_marks, float *distrib_minimum, float *distrib_maximum ){

   fprintf(stdout, "E Q U U S\n");
   int i = 0;
   float tmp_counter = 0.0;

   for(i = 0; i < distrib_size; i++){
      distrib[i] = 0.0;
      if(i < high){
         dist_marks[i] = 0.0;
      }else{
         dist_marks[i]  = 1.0;
      }
   }


   float equal_prob = 1.0;
   if(distrib_size > 0){

      equal_prob = 1.0/distrib_size;
   }else{
      equal_prob = 1.0;
   }

   if((high - low) > 0){
      equal_prob = 1.0/(high - low);
   }

   *distrib_maximum = equal_prob;
   *distrib_minimum = equal_prob;
   for(i = low; i < high; i++){
      distrib[i] = equal_prob;
      tmp_counter += distrib[i];
      dist_marks[i] = tmp_counter;
      //fprintf(stdout, "2mark: %i, acuu: %f, adder: %f\n", dist_marks[j], tmp_counter, distrib[i]);
   }
   //dist_marks[20] = high;

}


void makeDistriByParting(const int distrib_size, const int low, const int high, const float center_fact, const float divisor, float *distrib, float *dist_marks, float *distrib_minimum, float *distrib_maximum ){

   int number = high < low ? low-high : high-low;
   int distrib_center_scalar = (int) roundf(center_fact*number);
   float block = 1.0;
   int counter = 0;
   float values[number];
   float preliminary_sum = 0.0;
   float tmp_counter = 0.0;

   int i;
   for(i = 0; i < distrib_size; i++){
      distrib[i] = 0.0;
      if(i < high){
         dist_marks[i] = 0.0;
      }else{
         dist_marks[i]  = 1.0;
      }
   }

   values[counter] = block;
   counter++;
   while(counter < number){

      block *= divisor;

      values[counter - 1] = block;
      values[counter] = block;
      counter++;
   }

   for(i = 0; i < number; i++){
      preliminary_sum += values[i];
   }

   if(preliminary_sum != 1.0 && preliminary_sum != 0.0){
      preliminary_sum = 1.0/preliminary_sum;
      for(i = 0; i < number; i++){
         values[i] *= preliminary_sum;
      }
   }

   distrib[distrib_center_scalar] = values[0];
   i = 1;
   int distancer = 1;
   int distancel = 1;

   while(i < number){
      //fprintf(stdout, "preA %i %f\n", i, values[i]);
      if(distrib_center_scalar + distancer < number){
         distrib[distrib_center_scalar+distancer] = values[i];
         distancer++;
         i++;
         //fprintf(stdout, "A %i %f %i\n", i, values[i], distancel+distancer);
      }
      //fprintf(stdout, "B %i %f\n", i, values[i]);
      if(distrib_center_scalar - distancel >= 0){
         distrib[distrib_center_scalar-distancel] = values[i];
         distancel++;
         i++;
         //fprintf(stdout, "C %i %f %i\n", i, values[i], distancel+distancer);
      }
   }

   //int j = 0;
   *distrib_maximum = 0.0;
   *distrib_minimum = 1.0;
   //for(i = low; i < high; i++){
   for(i = 0; i < distrib_size; i++){
      if(distrib[i] > *distrib_maximum){
         *distrib_maximum = distrib[i];
      }
      if(distrib[i] < *distrib_minimum && distrib[i] > 0.0){
         *distrib_minimum = distrib[i];
      }
      tmp_counter += distrib[i];
      dist_marks[i] = tmp_counter;
      //fprintf(stdout, "%f\n", tmp_counter);
      //fprintf(stdout, "2mark: %i, acuu: %f, adder: %f\n", dist_marks[j], tmp_counter, distrib[i]);
      //fprintf(stdout, "%i\n", dist_marks[j]);
   }
   //fprintf(stdout, "%i\n", dist_marks[20]);

}

void makeDistriByUmverteilung(const int distrib_size, const int low, const int high, const float center_fact, const float part, float *distrib, float *dist_marks, float *distrib_minimum, float *distrib_maximum ){

   //int number = high -low;
   int number = high < low ? low-high : high-low;
   int distrib_center_scalar = (int) roundf(center_fact*number);
   float start = 1.0;
   int counter = 0;
   float values[number];
   float tmp_counter = 0.0;
   float co_part = 1.0 - part;

   if(number > 0){
      start = 1.0/number;
   }

   int i;
   for(i = 0; i < distrib_size; i++){
      distrib[i] = 0.0;
      if(i < high){
         dist_marks[i] = 0.0;
      }else{
         dist_marks[i]  = 1.0;
      }
   }

   while(counter < number){

      values[counter] = start + (number - (counter+1)) * start * part;
      start *= co_part;
      counter++;

   }

   distrib[distrib_center_scalar] = values[0];
   i = 1;
   int distancer = 1;
   int distancel = 1;

   while(i < number){
      //fprintf(stdout, "preA %i %f\n", i, values[i]);
      if(distrib_center_scalar + distancer < number){
         distrib[distrib_center_scalar+distancer] = values[i];
         distancer++;
         i++;
         //fprintf(stdout, "A %i %f %i\n", i, values[i], distancel+distancer);
      }
      //fprintf(stdout, "B %i %f\n", i, values[i]);
      if(distrib_center_scalar - distancel >= 0){
         distrib[distrib_center_scalar-distancel] = values[i];
         distancel++;
         i++;
         //fprintf(stdout, "C %i %f %i\n", i, values[i], distancel+distancer);
      }
   }

   //int j = 0;
   *distrib_maximum = 0.0;
   *distrib_minimum = 1.0;
   //for(i = low; i < high; i++){
   for(i = 0; i < distrib_size; i++){
      if(distrib[i] > *distrib_maximum){
         *distrib_maximum = distrib[i];
      }
      if(distrib[i] < *distrib_minimum && distrib[i] > 0.0){
         *distrib_minimum = distrib[i];
      }
      tmp_counter += distrib[i];
      dist_marks[i] = tmp_counter;
      //fprintf(stdout, "%f\n", tmp_counter);
      //fprintf(stdout, "2mark: %i, acuu: %f, adder: %f\n", dist_marks[j], tmp_counter, distrib[i]);
      //fprintf(stdout, "%i\n", dist_marks[j]);
   }
   //fprintf(stdout, "%i\n", dist_marks[20]);

}

//float distribs_get_distrib(const int index){
//
//	if( index >= 0 && index < distrib_size){
//		return distrib[index];
//	}else{
//		fprintf(stdout, "distribs_get_distrib: wrong index = %i\n", index);
//		return 0.0;
//	}
//
//}
//
//float distribs_get_dist_marks(const int index){
//
//	if( index >= 0 && index < distrib_size){
//		return dist_marks[index];
//	}else{
//		fprintf(stdout, "distribs_get_dist_marks: wrong index = %i\n", index);
//		return 0.0;
//	}
//
//}
//
//float distribs_get_distrib_maximum(){
//
//	return distrib_maximum;
//
//}
//
//float distribs_get_distrib_minimum(){
//
//	return distrib_minimum;
//
//}
//
//float *distribs_get_dist_marks_pointer(){
//
//	return dist_marks;
//
//}
