// Copyright 2015, Jean Rene Dawin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version. See License-GPL-2.txt.

#include "escwyp_globals_extern.h"
#include "escwyp_out_jack.h"
#include "escwyp_misch_extern.h"
#include "escwyp_out_txt_extern.h"
#include "escwyp_main.h"
#include <stdlib.h>
#include <string.h>

int escwypInitJack()
{
   /******** THE ZOUND aka jack ******/
   /**********************************/

   const char **ports;

   jack_set_error_function (escwyp_jack_error );

   const char *server_name = "default";
   jack_options_t options = JackNoStartServer;

   client = jack_client_open("escwyp", options, NULL, server_name); 
   int reti = 1;
   if(client == NULL) {
      fprintf (stderr, "jack server not running?\n");
      reti = -1;
   }else{
       fprintf(stdout, "The JACK Server = %x\n", client);

       jack_set_process_callback(client, process, 0);
       jack_set_sample_rate_callback(client, escwyp_jack_srate , 0);
       jack_set_buffer_size_callback(client, escwyp_jack_bufsize_changed,0);
       jack_on_shutdown(client, escwyp_jack_shutdown , 0);
       buffer_length_jack = jack_get_buffer_size(client);
       create_buffers(buffer_length_jack);
       printf ("engine buff lengthe: %lu )\n", buffer_length_jack);
       samp_rate_jack = jack_get_sample_rate(client);
       printf("engine sample rate: %lu\n", samp_rate_jack);

       output_port = jack_port_register(client, "output", 
               JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);

       if (jack_activate (client)) {
           fprintf(stderr, "cannot activate client");
       }

       if ((ports = jack_get_ports (client, NULL, NULL, 
                       JackPortIsPhysical|JackPortIsInput)) == NULL) {
           fprintf(stderr, "Cannot find any physical playback ports\n");
       }
       int i = 0;
       while(ports[i] != NULL){
           if (jack_connect (client, jack_port_name (output_port), ports[i])) {
               fprintf (stderr, "cannot connect output ports\n");
           }
           i++;
       }

       free(ports);
       reti = 0;
   }
   return reti;
}

int process(jack_nframes_t nframes, void *arg)
{

   sample_t *out = jack_port_get_buffer(output_port, nframes);

   //if(essen_ja == 1){

   //   memcpy(out, buff_L + gegessen*nframes, nframes*sizeof(sample_t));
   //   gegessen++;
   //   num_fills_proc--;
   //   if(num_fills_proc <= 0){
   //      buff_leere = 1;
   //      essen_ja = 0;
   //      gegessen = 0;
   //      faehnchen = 0;
   //   }
   //   if(gegessen == buff_parts){
   //      gegessen = 0;
   //      faehnchen = 0;
   //   }

   //   fprintf(stdout, "B (A) + %i nfp: %ld\n", gegessen, num_fills_proc);
   //}

   if(buff_leere == 0 && playsth == 0 && oszill_drawn == 0){

      fprintf(stdout, "OSZ %i\n", oszillcount); 
      buff_leere = 0;
      //makeOszilloskop();
      oszill_drawn = 1;
   }

   if(buff_leere == 1 && playsth == 0){

      memcpy(out, wartemal, sizeof(sample_t)*nframes);
      fprintf(stdout, "F W A R T E = %i\n", nframes); 
      buff_leere = 0;
      oszill_drawn = 0;
   }
   if(playsth == 0){

   	outTxtCopy(nframes, wartemal);
   }

   // When sound is going to be precessed we enter here  -------------
   if(playsth != 0){

      misch(nframes);
      memcpy(out, buff_L, nframes*sizeof(sample_t));
      outTxtCopy(nframes, buff_L);
      buff_leere = 2;

   }

   if(buff_leere == 2 && playsth < 0){
      int i;
      sample_t nframes_fact = 1.0;
      sample_t nframes_fact_subtract = 1.0;

      if(nframes > 1){
         nframes_fact_subtract = (sample_t) 1.0/(nframes-1);
      }

      for(i = 0; i < nframes; i++){
         out[i] *= nframes_fact;
         nframes_fact -= nframes_fact_subtract;
      }
      fprintf(stdout, "E N V E L O P E %i %f fact=%f\n", nframes, out[nframes-1], nframes_fact); 
      buff_leere = 1;
      playsth = 0;
   }



   return 0;      
}


int escwyp_jack_srate (jack_nframes_t nframes, void *arg)
{

   printf ("the sample rate is now %lu/sec\n", nframes);
   samp_rate_jack = nframes;
   return 0;
}

int escwyp_jack_bufsize_changed(jack_nframes_t nframes, void *arg)
{

   printf ("the  buffer_length_jack is now %lu\n", nframes);
   buffer_length_jack = nframes;
   return 0; 

}

void escwyp_jack_error (const char *desc)
{
   fprintf (stderr, "JACK error: %s\n", desc);
}

void escwyp_jack_shutdown_wrapper()
{
   escwyp_jack_shutdown(NULL);
}

void escwyp_jack_shutdown (void *arg)
{
    if(client != NULL) {
        jack_client_close (client);
    }
    fprintf (stderr, "JACK SHUTDOWN: %x\n", client);
    client = NULL;
}

