// Copyright 2015, Jean Rene Dawin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.

#include "escwyp_samplecast_patch_eet.h"
#include "escwyp_samplecast_extern.h"
#include "escwyp_samplecast_gui_elm_extern.h"
#include "escwyp_patch_select_extern.h" 
#include <string.h>

void samplecast_load_patch(const char *patch_name){

    fprintf(stdout, "III samplecast_load_patch\n");

    void *param_p = NULL;
    void *param_pp = NULL;
    char the_base_key[255];
    char the_key[255];
    the_base_key[0] = 47;
    the_base_key[1] = 0;
    strcat(the_base_key, "samplecast/"); 
    strcat(the_base_key, patch_name);

    // VORFAKTS----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/samplecast_vorfakts");

    param_pp = escwyp_patch_load_param(param_p, the_key);
    if(param_pp){
       float *vorfakts = (float *) param_pp;
       int i;
       //fprintf(stdout, "pp %x p %x v %x samplecast_vorfakts = ", param_pp, param_p, vorfakts);
       for(i = 0; i < 10; ++i){
           samplecast_set_vorfakts(i, vorfakts[i]);
           fprintf(stdout, "%f, ", vorfakts[i]);
           samplecast_register_sliders_set(i, vorfakts[i]);
       }
       fprintf(stdout, "\n, %s\n", the_key);
       free(param_pp);
       fprintf(stdout, "after free vorfakts\n");
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

    // INIT VALUE----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/samplecast_init_value");

    param_pp = escwyp_patch_load_param(param_p, the_key);
    if(param_pp){
       float *init_value = (float *) param_pp;
       
       samplecast_set_init_value(*init_value);
       fprintf(stdout, "samplecast_init_value = %f, %s\n", *init_value, the_key);
       samplecast_init_value_slider_set(*init_value);
       free(init_value);
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

    // MODE----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/samplecast_mode");

    int mode_tmp = 0; //needed for MODE OPT, see below 
    if(param_pp){
       param_pp = escwyp_patch_load_param(param_p, the_key);
       int *mode = (int *) param_pp;
       samplecast_set_mode(*mode);
       fprintf(stdout, "samplecast_mode = %i, %s\n", *mode, the_key);
       samplecast_mode_radios_set(*mode);
       mode_tmp = *mode;
       free(mode);
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

    // MODE OPT----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/samplecast_mode_opt");

    param_pp = escwyp_patch_load_param(param_p, the_key);
    if(param_pp){
       int *mode_opt = (int *) param_pp;
       samplecast_set_mode_opt(*mode_opt);
       fprintf(stdout, "samplecast_mode_opt = %i, %s\n", *mode_opt, the_key);
       if(mode_tmp == 0){
           samplecast_register_select_radios_set(*mode_opt);
       }
       if(mode_tmp == 2){
           float mode_opt_float = (float) *mode_opt;
           samplecast_magnify_slider_set(mode_opt_float);
       }
       free(mode_opt);
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

    samplecast_refresh_gui_elems();
}

void samplecast_save_patch(const char *patch_name){

    fprintf(stdout, "samplecast_save_patch\n");
    float vorfakts[10]; 
    float init_value = 0.3; 
    int mode = 0; 
    int mode_opt = 0; 

    char id_string[255];
    strcpy(id_string, "/PATCH_NAME/samplecast/");
    strcat(id_string, patch_name);
    escwyp_patch_save_param(patch_name, (1+strlen(patch_name))*sizeof(char), id_string);
    fprintf(stdout, "%s = %s\n", id_string, patch_name);

    char the_base_key[255];
    char the_key[255];
    the_base_key[0] = 47;
    the_base_key[1] = 0;
    strcat(the_base_key, "samplecast/"); 
    strcat(the_base_key, patch_name);


    int i;
    for(i = 0; i < 10; ++i){
        vorfakts[i] = samplecast_get_vorfakts(i);
    }
    strcpy(the_key, the_base_key);
    strcat(the_key, "/samplecast_vorfakts");
    escwyp_patch_save_param(vorfakts, 10*sizeof(float), the_key);
    fprintf(stdout, "%s\n", the_key);

    init_value = samplecast_get_init_value();
    strcpy(the_key, the_base_key);
    strcat(the_key, "/samplecast_init_value");
    escwyp_patch_save_param(&init_value, sizeof(float), the_key);
    fprintf(stdout, "%s\n", the_key);

    mode = samplecast_get_mode();
    strcpy(the_key, the_base_key);
    strcat(the_key, "/samplecast_mode");
    escwyp_patch_save_param(&mode, sizeof(int), the_key);
    fprintf(stdout, "%s %i\n", the_key, mode);

    mode_opt = samplecast_get_mode_opt();
    strcpy(the_key, the_base_key);
    strcat(the_key, "/samplecast_mode_opt");
    escwyp_patch_save_param(&mode_opt, sizeof(int), the_key);
    fprintf(stdout, "%s %i\n", the_key, mode_opt);

}
