// Copyright 2015, Jean Rene Dawin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version. See License-GPL-2.txt.

#include "escwyp_sine_grass_patch_eet.h"
#include "escwyp_sine_grass_extern.h"
#include "escwyp_sine_grass_gui_elm_extern.h"
#include "escwyp_patch_select_extern.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void sine_grass_load_patch(const char *patch_name){

    fprintf(stdout, "III sine_grass_load_patch\n");

    void *param_p = NULL;
    void *param_pp = NULL;
    char the_base_key[255];
    char the_key[255];
    the_base_key[0] = 47;
    the_base_key[1] = 0;
    strcat(the_base_key, "sine_grass/"); 
    strcat(the_base_key, patch_name);

    // SINE_GRASS_FREQ
    strcpy(the_key, the_base_key);
    strcat(the_key, "/grass_freq");

    param_pp = escwyp_patch_load_param(param_p, the_key);

    if(param_pp){
       float *grass_freq = (float *) param_pp;
       sine_grass_params_sine_grass_freq_set(*grass_freq);
       sine_grass_freq_slider_set(*grass_freq);
    
       fprintf(stdout, "%s %f\n", the_key, *grass_freq);
       free(param_pp);
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //----------------

    // SINE GRASS HILL FRACTION----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/grass_hill_fraction");

    param_pp = escwyp_patch_load_param(param_p, the_key);
    if(param_pp){
       float *grass_hill_fraction = (float *) param_pp;
       sine_grass_set_grass_hill_fraction(*grass_hill_fraction);
       sine_grass_grass_hill_fraction_slider_set(*grass_hill_fraction);

       fprintf(stdout, "%s %f\n", the_key, *grass_hill_fraction);
       free(param_pp);
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

    // GRASS MOWED----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/grass_mowed");
    param_pp = escwyp_patch_load_param(param_p, the_key);
	   
    if(param_pp){
       int *grass_mowed = (int *) param_pp;
       sine_grass_set_grass_mowed(*grass_mowed);
       sine_grass_grass_mowed_check_set(*grass_mowed);
              
       fprintf(stdout, "%s %i\n", the_key, *grass_mowed);
       free(param_pp);
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

    // LAWN LENGTH VARIATION FRAC----------------------
    strcpy(the_key, the_base_key);
    strcat(the_key, "/lawn_length_variation_frac");
    param_pp = escwyp_patch_load_param(param_p, the_key);
	   
    if(param_pp){
       float *lawn_length_variation_frac = (float *) param_pp;
       sine_grass_set_lawn_length_variation_frac(*lawn_length_variation_frac);
       sine_grass_lawn_length_variation_frac_slider_set(*lawn_length_variation_frac);
              
       fprintf(stdout, "%s %f\n", the_key, *lawn_length_variation_frac);
       free(param_pp);
    }else{
       fprintf(stdout, "%s not there. Ohh.\n", the_key);
    }
    //--------------------------------

}

void sine_grass_save_patch(const char *patch_name){

    fprintf(stdout, "sine_grass_save_patch\n");
    float grass_freq = 0.0;
    float grass_hill_fraction = 0.0;
    int grass_mowed = 0;
    float lawn_length_variation_frac = 0.0;

    char id_string[255];
    strcpy(id_string, "/PATCH_NAME/sine_grass/");
    strcat(id_string, patch_name);
    escwyp_patch_save_param(patch_name, (1+strlen(patch_name))*sizeof(char), id_string);
    fprintf(stdout, "%s = %s\n", id_string, patch_name);

    char the_base_key[255];
    char the_key[255];
    the_base_key[0] = 47;
    the_base_key[1] = 0;
    strcat(the_base_key, "sine_grass/"); 
    strcat(the_base_key, patch_name);


    grass_freq = sine_grass_params_sine_grass_freq_get();
    strcpy(the_key, the_base_key);
    strcat(the_key, "/grass_freq");
    escwyp_patch_save_param(&grass_freq, sizeof(float), the_key);
    fprintf(stdout, "%s %f\n", the_key, grass_freq);

    grass_hill_fraction = sine_grass_get_grass_hill_fraction();
    strcpy(the_key, the_base_key);
    strcat(the_key, "/grass_hill_fraction");
    escwyp_patch_save_param(&grass_hill_fraction, sizeof(float), the_key);
    fprintf(stdout, "%s %f\n", the_key, grass_hill_fraction);

    grass_mowed = sine_grass_get_grass_mowed();
    strcpy(the_key, the_base_key);
    strcat(the_key, "/grass_mowed");
    escwyp_patch_save_param(&grass_mowed, sizeof(int), the_key);
    fprintf(stdout, "%s %i\n", the_key, grass_mowed);

    lawn_length_variation_frac = sine_grass_get_lawn_length_variation_frac();
    strcpy(the_key, the_base_key);
    strcat(the_key, "/lawn_length_variation_frac");
    escwyp_patch_save_param(&lawn_length_variation_frac, sizeof(float), the_key);
    fprintf(stdout, "%s %f\n", the_key, lawn_length_variation_frac);

}

