// Copyright 2013-2015, Jean Rene Dawin
//
// This Library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 2.1 of the License, or
// (at your option) any later version.
//
// This Library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this Library.  If not, see <http://www.gnu.org/licenses/>.

#include <stdio.h>
#include <stdlib.h>
#include "escwyp_sine_robin.h"
//#include "escwyp_sine_robin_gui_elm_extern.h"
#include "escwyp_distribs_extern.h"
//#include "escwyp_klavier_extern.h"


void sine_robin_default_params(struct sine_robin_params *the_params)
{
   
   the_params->sine_robin_pos = 0.0;
   the_params->sine_robin_freq = 0.0;
   the_params->sine_robin_distrib_center = 0.5;

}

void print_params_sine_robin(struct sine_robin_params *the_params)
{

   fprintf(stdout, "sine_robin_pos = %f\n", the_params->sine_robin_pos);
   fprintf(stdout, "sine_robin_freq = %f\n", the_params->sine_robin_freq);
   fprintf(stdout, "sine_robin_distrib_center = %f\n", the_params->sine_robin_distrib_center);

}

void sine_robin_refresh_parameters()
{

   // the "0" means don't call makeDistribImage
   sine_robin_update_distrib();

}

void sine_robin_set_params(struct sine_robin_params *params)
{

   the_params = params;

}
void sine_robin_set_freq(const float freq)
{

   the_params->sine_robin_freq = freq;

}
void sine_robin_init_values_form_klavier(float note_id, float id_low, float id_high)
{

   the_params->sine_robin_freq = 0.0;
   the_params->sine_robin_pos = 0.0;
   if(!self_control){
	   the_params->sine_robin_distrib_center = 0.001 + (note_id - id_low)/(id_high - id_low)*(1.0 - 0.001);
   }
 
}

void sine_robin_init_values(const float a_distrib_center)
{

   the_params->sine_robin_freq = 0.0;
   the_params->sine_robin_pos = 0.0;
   the_params->sine_robin_distrib_center = a_distrib_center;

}


void sine_robin_update_distrib()
{

   switch(sine_robin_get_distrib_function()){

      case 0: {
                 makeEqualDistribution(sine_robin_distrib_size, sine_robin_distrib_low, sine_robin_distrib_high, sine_robin_distrib, sine_robin_dist_marks, &sine_robin_distrib_minimum, &sine_robin_distrib_maximum);
              }
              break;
      case 1: {
                 sine_robin_distrib_height_fine =  sine_robin_distrib_height/200;
                 makeDistri(sine_robin_distrib_size, sine_robin_distrib_low, sine_robin_distrib_high, the_params->sine_robin_distrib_center, sine_robin_distrib_height-(sine_robin_distrib_height_fine_frac*sine_robin_distrib_height_fine), sine_robin_distrib, sine_robin_dist_marks, &sine_robin_distrib_minimum, &sine_robin_distrib_maximum);
              }
              break;
      case 2: {
                 sine_robin_distrib_height_fine =  (1.0-sine_robin_distrib_height)/200;
                 makeDistriByParting(sine_robin_distrib_size, sine_robin_distrib_low, sine_robin_distrib_high, the_params->sine_robin_distrib_center, sine_robin_distrib_height+(sine_robin_distrib_height_fine_frac*sine_robin_distrib_height_fine), sine_robin_distrib, sine_robin_dist_marks, &sine_robin_distrib_minimum, &sine_robin_distrib_maximum);

              }
              break;
      case 3: {
                 sine_robin_distrib_height_fine =  sine_robin_distrib_height/200;
                 makeDistriByUmverteilung(sine_robin_distrib_size, sine_robin_distrib_low, sine_robin_distrib_high, the_params->sine_robin_distrib_center, sine_robin_distrib_height-(sine_robin_distrib_height_fine_frac*sine_robin_distrib_height_fine), sine_robin_distrib, sine_robin_dist_marks, &sine_robin_distrib_minimum, &sine_robin_distrib_maximum);
              }
              break;

   }

}

//void makeSineRobin2(const int number_of_samples){
//
//   makeSineRobin(number_of_samples, 2048, sine_robin_dist_marks); 
//
//}
// This function is called when button "Key3" is pressed. ----------------
void makeSineRobin(const int number_of_samples, float *buff_L, const unsigned int srate)
{

   //srand(time(0));
   unsigned int sample_rate = srate;
   if(sample_rate == 0){
      sample_rate = 48000;
   }

   int current_samples = 0;
   float rnda = 0.0;
   //float distrib_runner = 0.0;
   //float bisection_parts[21] =  {0.0, 0.05, 0.1, 0.15, 0.2, 0.25, 0.3, 0.35, 0.4, 0.45, 0.5, 0.55, 0.6, 0.65, 0.7, 0.75, 0.8, 0.85, 0.9, 0.95, 1.0};
   //int bisection_indices[6] = {4, 2, 1, 1, 1, 1};

   //int i = 0;
   //for(i = 20; i < 40; i++){
   //   fprintf(stdout, "sine_robin_dist_marks[%i] = %i\n", i-20, sine_robin_dist_marks[i-20]);
   //}
   while( current_samples < number_of_samples ){

      the_params->sine_robin_pos += the_params->sine_robin_freq / sample_rate;
      float sine_robin_pos_rounded = roundf(the_params->sine_robin_pos); 
      if(fabs(the_params->sine_robin_pos - sine_robin_pos_rounded) <= 0.001){
         //fprintf(stdout, "the_params->sine_robin_pos: %f\n", the_params->sine_robin_pos);
         the_params->sine_robin_pos = 0.0; // chek!
      }

      buff_L[current_samples]  = (float) sinf(2.0 * M_PI * the_params->sine_robin_pos);

      rnda = (float) random()/RAND_MAX;

      int bisection_length = 1024;
      int selected_pos = 1024;
      int increment_sign = 0;
      //i = 0;
      //fprintf(stdout, "BEFORE\n");
      while(bisection_length >= 2){
         bisection_length = bisection_length/2;
         if(rnda <= sine_robin_dist_marks[selected_pos]){
            selected_pos -= bisection_length;
            increment_sign = -1;
            //i++;
         }else{
            selected_pos += bisection_length;
            increment_sign = 1;
            //i++;
         } 
      }      
      if(increment_sign < 0){
         if(rnda <= sine_robin_dist_marks[selected_pos - 1]){
            selected_pos --;
            //i++;
         }
      }else{
         if(rnda > sine_robin_dist_marks[selected_pos]){
            selected_pos ++;
            //i++;
         }
      }

      //fprintf(stdout, "AFTER %i\n", i);

      the_params->sine_robin_freq = 1.0*selected_pos;
      //fprintf(stdout, "%f\n", the_params->sine_robin_freq);

      //fprintf(stdout, "BFTER %i\n", i-lower_bound);
      //fprintf(stdout, "2 the_params->sine_robin_freq = %f\n", the_params->sine_robin_freq);
      //the_params->sine_robin_freq = lower_bound + rnda*(upper_bound - lower_bound);

      current_samples++;

   }
}

//Getters/Setters

float sine_robin_get_distrib(const int index)
{

	return sine_robin_distrib[index];

}

float sine_robin_get_dist_marks(const int index)
{

	return sine_robin_dist_marks[index];

}

float sine_robin_get_distrib_center()
{

	return the_params->sine_robin_distrib_center;

}

float sine_robin_get_distrib_height()
{

	return sine_robin_distrib_height;

}

float sine_robin_get_distrib_height_fine()
{

	return sine_robin_distrib_height_fine;

}

float sine_robin_get_distrib_height_fine_frac()
{

	return sine_robin_distrib_height_fine_frac;

}

float sine_robin_get_distrib_maximum()
{

	return sine_robin_distrib_maximum;

}

float sine_robin_get_distrib_minimum()
{

	return sine_robin_distrib_minimum;

}

int sine_robin_get_distrib_low()
{

	return sine_robin_distrib_low;

}

int sine_robin_get_distrib_high()
{

	return sine_robin_distrib_high;

}

int sine_robin_get_distrib_function()
{

	return sine_robin_distrib_function;

}

void sine_robin_set_distrib_center(const float value)
{

	the_params->sine_robin_distrib_center = value;

}

void sine_robin_set_distrib_height(const float value)
{

	sine_robin_distrib_height = value;

}

void sine_robin_set_distrib_height_fine(const float value)
{

	sine_robin_distrib_height_fine = value;

}

void sine_robin_set_distrib_height_fine_frac(const float value)
{

	sine_robin_distrib_height_fine_frac = value;

}

void sine_robin_set_distrib_maximum(const float value)
{

	sine_robin_distrib_maximum = value;

}

void sine_robin_set_distrib_minimum(const float value)
{

	sine_robin_distrib_minimum = value;

}

void sine_robin_set_distrib_low(const int value)
{

	sine_robin_distrib_low = value;

}

void sine_robin_set_distrib_high(const int value)
{

	sine_robin_distrib_high = value;

}

void sine_robin_set_distrib_function(const int value)
{

	sine_robin_distrib_function = value;

}
void sine_robin_set_self_control(const int value)
{

    self_control = value;

}
int sine_robin_get_self_control()
{

    return self_control;

}

