// Copyright 2013-2015, Jean Rene Dawin
//
// This Library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 2.1 of the License, or
// (at your option) any later version.
//
// This Library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this Library.  If not, see <http://www.gnu.org/licenses/>.

#ifndef ESCWYP_SINE_ROBIN_H
#define ESCWYP_SINE_ROBIN_H
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "escwyp_sine_robin_params.h"
//#include "escwyp_globals_extern.h"
//#include "escwyp_distribs_extern.h"

void sine_robin_default_params(struct sine_robin_params *the_params);
void print_params_sine_robin(struct sine_robin_params *the_params);

static struct sine_robin_params default_params = { 0.0, 0.0, 0.5 };

static struct sine_robin_params *the_params = &default_params;

// the center and height/fine parameters for the distribution functions
static int sine_robin_distrib_size = 2048;
static float sine_robin_distrib[2048];
static float sine_robin_dist_marks[2048];
 
//static float sine_robin_distrib_center = 0.5;
//static int sine_robin_distrib_center_scalar = 0; 
static float sine_robin_distrib_height = 0.5;
static float sine_robin_distrib_height_fine  = 0.0;
static float sine_robin_distrib_height_fine_frac = 0.0;
// the lower and upper bounds for makeSineRobin
static int sine_robin_distrib_low = 20;
static int sine_robin_distrib_high = 2048;
// the value of the radio buttons selcting the distribution function
static int sine_robin_distrib_function = 2;

//// the maximum and minimum values of the current distribution  
static float sine_robin_distrib_maximum = 1.0;
static float sine_robin_distrib_minimum = 0.0;

static int self_control = 0;

// the per sample frequency of the makeSineRobin function
//static float sine_robin_freq = 0.0;
//static float sine_robin_pos = 0.0;
 

void sine_robin_update_distrib();
void sine_robin_refresh_parameters();
void sine_robin_set_params(struct sine_robin_params *params);
void sine_robin_set_freq(const float freq);
void sine_robin_init_values_form_klavier(float note_id, float id_low, float id_high);
void sine_robin_init_values(const float a_distrib_center);
void makeSineRobin(const int number_of_samples, float *buff_L, const unsigned int srate);
//void makeSineRobin2(const int number_of_samples);

// Getters/Setters
float sine_robin_get_distrib(const int index);
float sine_robin_get_dist_marks(const int index);
float sine_robin_get_distrib_center();
float sine_robin_get_distrib_height();
float sine_robin_get_distrib_height_fine();
float sine_robin_get_distrib_height_fine_frac();
float sine_robin_get_distrib_maximum();
float sine_robin_get_distrib_minimum();
int sine_robin_get_distrib_low();
int sine_robin_get_distrib_high();
int sine_robin_get_distrib_function();

void sine_robin_set_distrib_center(const float value);
void sine_robin_set_distrib_height(const float value);
void sine_robin_set_distrib_height_fine(const float value);
void sine_robin_set_distrib_height_fine_frac(const float value);
void sine_robin_set_distrib_maximum(const float value);
void sine_robin_set_distrib_minimum(const float value);
void sine_robin_set_distrib_low(const int value);
void sine_robin_set_distrib_high(const int value);
void sine_robin_set_distrib_function(const int value);
void sine_robin_set_self_control(const int value);
int sine_robin_get_self_control();

#endif
