// Copyright 2015, Jean Rene Dawin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version. See License-GPL-2.txt.
//
// gcc -c -o libsamplot.so libsamplot.c -lsndfile

#include <stdio.h>
#include <sndfile.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

static long write_samples(const int num, float *samps, SNDFILE *file)
{
   long frames = 0; 
   if(file){
      frames = sf_writef_float(file, samps, num);
   }
   return frames;
} 

// sndfile  = the SNDFILE pointer to the open file
int samplot_close_file(const void *sndfile)
{
   int ret = -1;
   if(sndfile != NULL){
      SNDFILE *sf = (SNDFILE *) sndfile;
      sf_write_sync(sf);
      sf_close(sf);
      ret = 0;
   }
   return ret;
}

// buff     = the buffer with floats that are written to file named >name<
// num      = the number of floats in >buff<
// sndfile  = the SNDFILE pointer to the open file
int samplot_append_to_file(float *buff, const int num, const void *sndfile)
{
   int ret = -1;
   int go = 0;
   if(buff != NULL){
      go += 2;
   }
   if(num > 0){
      go += 4;
   }
   if(sndfile != NULL){
      go += 8;
   }
   if(go == 14){
      SNDFILE *sf = (SNDFILE *) sndfile;
      long writtenFrames = write_samples(num, buff, sf);
      if (writtenFrames != num){
         fprintf(stderr, "nicht genuegend Frames\n");
         ret = -2;
      }else{
         ret = 0;
         fprintf(stdout, "Appending NUM_SAMPLES %li sf=%x\n", writtenFrames, sf);
         sf_write_sync(sf);
      }
   }else{
      fprintf(stderr, "Fehler beim Appenden. go=%i\n", go);
      ret = -3;
   }  
   return ret;
}


// buff  = the buffer with floats that are written to file named >name<
// num   = the number of floats in >buff<
// name  = the path without file extension e.g. "/tmp/soundfiles/trumpet"
// type  = the file extension/type e.g "wav"
// srate = the sample rate to use e.g. 48000
// sndfile = void pointer, where the SNDFILE pointer will be stored for the caller for later use
int samplot_write_to_file(float *buff, const int num, char *name, char *type, const int srate, void **sndfile)
{
    int ret = -1;
    int num_samples = 0;
    int sr = 48000;
    int go = 0;
    if(srate > 0){
	go += 1;
    }
    if(buff != NULL){
	go += 2;
    }
    if(num > 0){
	go += 4;
    }
    if(name != NULL){
	go += 8;
    }
    if(strncmp(type, "wav", 3) == 0){
	go += 16;
    }
    if(go == 31){
       char sound_name[255];
       sound_name[0] = '\0';
       strncat(sound_name, name, 250);
       strcat(sound_name, ".");
       strncat(sound_name, type, 3);
       fprintf(stdout, "SOUND_NAME %s\n", sound_name);

       SF_INFO info_mc;
       info_mc.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
       info_mc.channels = 1;
       info_mc.samplerate = srate;

       SNDFILE *sndFile_mc = sf_open(sound_name, SFM_WRITE, &info_mc);
       if (sndFile_mc != NULL){
          *sndfile = sndFile_mc;
	
          long writtenFrames = write_samples(num, buff, sndFile_mc);
          if (writtenFrames != num){
              fprintf(stderr, "nicht genuegend Frames\n");
	      ret = -2;
          }else{
	      ret = 0;
	  }
          fprintf(stdout, "NUM_SAMPLES %li\n", writtenFrames);

          sf_write_sync(sndFile_mc);
          //sf_close(sndFile_mc);

       }else{
          fprintf(stderr, "Fehler beim Dateioeffnen: %s\n", sf_strerror(sndFile_mc));
	  ret = -3;
       }  
    }else{
           fprintf(stderr, "Nothing worked. Poor input parameter. We feel with you. (%i)\n", go);
	   ret = -4;
	//switch(go){
	//   case 0: {
        //  	     fprintf(stderr, "Nothing worked. Poor input parameter. We feel with you.\n");
	//	     break;
	//	   }
	//   case 1: {
        //  	     fprintf(stderr, "srate is greater 0. Great.\n");
	//	     break;
	//	   }
	//   case 2: {
        //  	     fprintf(stderr, "buff is not NULL. Big.\n");
	//	     break;
	//	   }
	//   case 4: {
        //  	     fprintf(stderr, "num is greater 0. Formidable.\n");
	//	     break;
	//	   }
	//   case 8: {
        //  	     fprintf(stderr, "name is not NULL. Excellent.\n");
	//	     break;
	//	   }
	//   case 16: {
        //  	     fprintf(stderr, "type is wav. Superb.\n");
	//	     break;
	//	   }
	//   case 3: {
        //  	     fprintf(stderr, "srate is greater 0. Great.\n");
        //  	     fprintf(stderr, "buff is not NULL. Big.\n");
	//	     break;
	//	   }
	//   case 5: {
        //  	     fprintf(stderr, "srate is greater 0. Great.\n");
        //  	     fprintf(stderr, "num is greater 0. Formidable.\n");
	//	     break;
	//	   }
	//   case 6: {
        //  	     fprintf(stderr, "buff is not NULL. Big.\n");
        //  	     fprintf(stderr, "num is greater 0. Formidable.\n");
	//	     break;
	//	   }
        // }
    }			
    return ret;
}
